#ifndef PW_SCENE_NODE_HPP
#define PW_SCENE_NODE_HPP

#include <pw/core/globals.hpp>
#include <pw/scene/component.hpp>

#include <string>
#include <vector>

namespace pw {

class traverser;

/**
 * @brief nodes represent the structure for the scene graph
 */
class node {
public:

    typedef shared_ptr<node> ref;
    typedef std::vector<ref> ref_array;
    typedef std::vector<node*> ptr_array;

    friend class component;

    //! standard c'tor
    node(const std::string& name = "");

    //! copy c'tor
    node(const node& node);


	//! wrapper for scripting interface
	std::shared_ptr<node> shared();

    //! cloning interface
    virtual node* clone(const unsigned short& copymode) const;

    std::string name() const;
    void set_name(const std::string &name);

    //! return list of parents
    inline const ptr_array& parents() const { return _parents; }

    //! add a child node
	std::shared_ptr<node> add_child(std::shared_ptr<node> anode);

    //! get the list of children
    inline const ref_array& children() const { return _children; }

    //! get number of children
    inline size_t child_count() const { return children().size(); }

    //! check if this node is a leaf node
    inline bool is_leaf() const { return children().empty(); }

	//! check if this is the root node
	inline bool is_root() const { return parents().empty(); }

	//! d'tor
    virtual ~node();

	//! list of components
    const component::array& components() const;

	//! add a node as parent node
	void add_parent(node *anode);

	//! visitor to collect data from the graph
	struct visitor {
		enum direction {
			up,
			down
		} direction;

		visitor(enum direction d = direction::down) : direction(d) {}

		virtual void enter(node* n) = 0;
		virtual void leave(node* n) = 0;
	};

	void visit(visitor &visitor);

	void ascend(visitor &visitor);

	void descend(visitor &visitor);

protected:

    void register_component(component* c);
    void unregister_component(component* c);

    ref_array _children;
    ptr_array _parents;

    component::array _components;

    std::string _name;
};

}

#endif
